<?php

namespace Drupal\commerce_epayco\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the standard checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "epayco_standard_checkout",
 *   label = "ePayco (Standard checkout)",
 *   display_label = "ePayco (Standard checkout)",
 *   modes = {
 *     "n/a" = @Translation("N/A"),
 *   },
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_epayco\PluginForm\OffsiteRedirect\StandardCheckoutForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "amex", "dinersclub", "discover", "jcb", "maestro", "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE
 * )
 */
class StandardCheckout extends OffsiteCheckoutBase {

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $remote_ref = $request->query->get('ref_payco');
    $info = $remote_ref ? $this->gatewayHandler->getReferenceRemoteData($remote_ref) : [];
    if (isset($info['data'])) {
      $x_cod_transaction_state = isset($info['data']['x_cod_transaction_state']) ? $info['data']['x_cod_transaction_state'] : '';
      $x_ref_payco = isset($info['data']['x_ref_payco']) ? $info['data']['x_ref_payco'] : '';
      $x_transaction_state = isset($info['data']['x_transaction_state']) ? $info['data']['x_transaction_state'] : '';
      $x_response_reason_text = isset($info['data']['x_response_reason_text']) ? $info['data']['x_response_reason_text'] : '';

      switch ($x_cod_transaction_state) {
        case 1:
          // Aceptada - Accepted.
        case 3:
          // Pendiente - Pending.
          $this->entityTypeManager->getStorage('commerce_payment')->create([
            'state' => $x_cod_transaction_state == 1 ? 'completed' : 'authorization',
            'amount' => $order->getTotalPrice(),
            'payment_gateway' => $this->entityId,
            'order_id' => $order->id(),
            'test' => $this->getMode() == 'test',
            'remote_id' => $x_ref_payco,
            'remote_state' => $x_transaction_state,
            'authorized' => REQUEST_TIME,
          ])->save();
          $this->messenger()->addMessage($this->t('Transaction @status by ePayco.', ['@status' => $x_transaction_state]));
          break;

        case 2:
          // Rechazada - Rejected.
        case 4:
          // Fallida - Failed.
          $this->messenger()->addMessage($this->t('Order was finished, but payment transaction failed. ePayco says: @status', ['@status' => $x_transaction_state]), 'error');
          $this->logger->log('error', $x_response_reason_text);
          break;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    $status = $request->get('status');
    $this->messenger()->addMessage($this->t('Payment @status on @gateway but may resume the checkout process here when you are ready.', [
      '@status' => $status,
      '@gateway' => $this->getDisplayLabel(),
    ]), 'error');
  }

}
